#!/usr/bin/python3
# https://www.tutorialspoint.com/python3/python_multithreading.htm

"""
Dos exemples d'execució del mateix codi en moments diferents:

jordi@ecat-debian:~/Documents/Clot2019-2020/registreCodVid19/dam/m09uf2/20200317_m09uf2_covid19$ python3 fils04.py 
Creació dels nous fils
Starting Thread-1
Starting Thread-2
Starting Thread-3
Emplenament de la cua
Esperant la cua buida
Thread-1 processing One
Thread-3 processing Two
Thread-2 processing Three
Thread-1 processing Four
Thread-3 processing Five
Moment de sortir dels fils
Esperant que tots els fils acabin la seva execució
Exiting Thread-3
Exiting Thread-2
Exiting Thread-1
Exiting Main Thread
jordi@ecat-debian:~/Documents/Clot2019-2020/registreCodVid19/dam/m09uf2/20200317_m09uf2_covid19$ python3 fils04.py 
Creació dels nous fils
Starting Thread-1
Starting Thread-2
Emplenament de la cua
Starting Thread-3
Esperant la cua buida
Thread-3 processing Two
Thread-2 processing One
Thread-3 processing Three
Thread-3 processing Four
Thread-2 processing Five
Moment de sortir dels fils
Esperant que tots els fils acabin la seva execució
Exiting Thread-1
Exiting Thread-3
Exiting Thread-2
Exiting Main Thread

"""

import queue
import threading
import time

exitFlag = 0

class myThread (threading.Thread):
   def __init__(self, threadID, name, q):
      threading.Thread.__init__(self)
      self.threadID = threadID
      self.name = name
      self.q = q
   def run(self):
      print ("Starting " + self.name)
      process_data(self.name, self.q)
      print ("Exiting " + self.name)

def process_data(threadName, q):
   while not exitFlag:
      queueLock.acquire()
      if not workQueue.empty():
         data = q.get()
         queueLock.release()
         print ("%s processing %s" % (threadName, data))
      else:
         queueLock.release()
         time.sleep(1)

threadList = ["Thread-1", "Thread-2", "Thread-3"]
nameList = ["One", "Two", "Three", "Four", "Five"]
queueLock = threading.Lock()
workQueue = queue.Queue(10)
threads = []
threadID = 1

# Create new threads
print ("Creació dels nous fils")
for tName in threadList:
   thread = myThread(threadID, tName, workQueue)
   thread.start()
   threads.append(thread)
   threadID += 1

# Fill the queue
queueLock.acquire()
print("Emplenament de la cua")
for word in nameList:
   workQueue.put(word)
queueLock.release()

# Wait for queue to empty
print("Esperant la cua buida")
while not workQueue.empty():
   pass

# Notify threads it's time to exit
print("Moment de sortir dels fils")
exitFlag = 1

# Wait for all threads to complete
print("Esperant que tots els fils acabin la seva execució")
for t in threads:
   t.join()
print ("Exiting Main Thread")
